<?php
/*
Plugin Name: AdvertWayCAPTCHA
Plugin URI: http://www.advertway.pl/
Description: Integruje system antyspamowo-reklamowy AdvertWayCAPTCHA z platformą Wordpress
Version: 1.0
Author: AdvertWay Team
Email: wydawcy@advertway.pl
Author URI: http://www.advertway.pl
Created: 2011-Oct-01 12:00 (EDT)

Copyright (c) 2010 by AdvertWay.pl

$Id: advertwayCaptcha.php,v 1.0 2011/10/01 12:00:00 by AdvertWay Team $
*/

class AdvertWayCaptchaWPPlugin
{
  var $wpmu;
  var $awcaptcha_opt;
  var $awcaptcha;
  var $awcaptchaHashSalt;
  var $savedError;
  
  /**
   * Constructor
   */
  function AdvertWayCaptchaWPPlugin()
  {
    $this->awcaptchaHashSalt = '5dvxmbipxy6tf4tlcevt9e1gedqsnm94';
    $this->wpmu = $this->setWPMU();
    $this->awcaptcha_opt = $this->getOptionsFromDatabase();
    $this->savedError = '';
  }
  
  /**
   * Creates instance of AdvertWayApi
   */
  function awCaptchaInit()
  {
    $this->awcaptcha = new AdvertWayApi($this->awcaptcha_opt['public_key'], $this->awcaptcha_opt['private_key']);
  }
  
  /**
   * Detecting WordPress MultiUser
   * @return integer
   */
  function setWPMU()
  {
    if(basename(dirname(__FILE__)) == "mu-plugins")
       return 1;
    elseif (basename(dirname(__FILE__)) == "plugins" && function_exists('is_site_admin'))
       return 2;
    return 0;
  }
  
  function getWPMU()
  {
    return $this->wpmu;
  }
  
  /**
   * Get options from database
   * @return mixed
   */
  function getOptionsFromDatabase()
  {
    if($this->wpmu == 1)
       return get_site_option('awcaptcha');
    else
       return get_option('awcaptcha');
  }
  
  /**
   * Delete awcaptcha preferences from database
   */
  function deletePreferences()
  {
    if($this->wpmu != 1)
      delete_option('awcaptcha');
  }
  
  /**
   * Includes stylesheet in typical pages
   */
  function includeStylesheet()
  {
    $path = WP_CONTENT_URL . '/plugins/advertwayCaptcha/advertwayCaptcha.css';

    if($this->wpmu == 1)
      $path = WP_CONTENT_URL . '/mu-plugins/advertwayCaptcha/advertwayCaptcha.css';

     echo '<link rel="stylesheet" type="text/css" href="' . $path . '" />';
  }
  
  /**
   * Includes login page styling
   */
  function includeRegistrationStylesheet()
  {
     if($this->awcaptcha_opt['registration'])
     {
        echo '<style type="text/css">
                #login {width: 410px !important;}
                #login .advertway_captcha{margin: 0 auto 15px auto; }
                #login .advertway_captcha a{border: none;}
              </style>';
     }
  }
  
  /**
   * Displays AdvertWayCAPTCHA on the registration form or in a Contact Form 7 form
   * @param mixed $errors array of errors from WordPress or array with keys type and name from contact form 7
   * @return string 
   */
  function displayCaptcha($errors)
  { 
    if(($this->awcaptcha_opt['registration'] && $errors['type'] != 'awcaptcha') || ($this->awcaptcha_opt['contact_form_7'] && $errors['type'] == 'awcaptcha'))
    {
      if($this->wpmu == 1)
      {
        $error = $errors->get_error_message('captcha');
        echo '<label for="verification">Weryfikacja:</label>';
        $html = $error ? '<p class="error">'.$error.'</p>' : '';
        $html = $html . $this->awcaptcha->getCaptchaWidget();
      }
      else
      {
        $html = $this->awcaptcha->getCaptchaWidget();
        if($errors['type'] == 'awcaptcha')
          $html = '<span><span class="wpcf7-form-control-wrap awcaptcha"></span>' . $html . '</span>';
        else
          $html = '<hr style="clear: both; margin-bottom: 1.5em; border: 0; border-top: 1px solid #999; height: 1px;" />' . $html;
      }
    }

    if($this->awcaptcha_opt['contact_form_7'] && $errors['type'] == 'awcaptcha')
      return $html;

    echo $html;
  }
  
  /**
   * Checks if user type in correct solution for AdvertWayCAPTCHA in registration or Contact Form 7 form
   * @global mixed $errors
   * @global mixed $wpcf7_contact_form
   * @param mixed $result
   * @param string $tag
   * @return mixed 
   */
  function checkCaptcha($result, $tag = '')
  {
    global $errors, $wpcf7_contact_form;

    if($tag != '') //Contact form 7
      $calling_form = 1;
    elseif(version_compare(get_bloginfo('version'), '2.5' ) < 0) //Registration form on WordPress older than 2.5
      $calling_form = 2;
    else //Registration form on WordPress 2.5 or newer
      $calling_form = 3;

    if($_POST['captcha_challenge_code'])
    {
      if(empty($_POST['captcha_solution']))
      {
        switch($calling_form)
        {
          case 1:
            $result['valid'] = false;
            $result['reason']['awcaptcha'] = $this->awcaptcha_opt['error_blank_cf7'];
            break;
          case 2:
            $result['blank_captcha'] = $this->awcaptcha_opt['error_blank'];
            break;
          case 3:
            $result->add('blank_captcha', $this->awcaptcha_opt['error_blank']);
            break;
        }
      }
      else
      {
        if(!$this->awcaptcha->validate())
        {
          switch($calling_form)
          {
            case 1:
                $result['valid'] = false;
                $result['reason']['awcaptcha'] = $this->awcaptcha_opt['error_incorrect_cf7'];
                break;
            case 2:
                $result['captcha_wrong'] = $this->awcaptcha_opt['error_incorrect'];
                break;
            case 3:
                $result->add('captcha_wrong', $this->awcaptcha_opt['error_incorrect']);
                break;
          }
        }
      }
    }

    return $result;
  }
  
  /**
   * Register JQuery reloading on successful AJAX submit
   */
  function enqueueJS()
  {
    if ($this->wpmu !=1)
        wp_enqueue_script('refreshCaptcha', WP_CONTENT_URL . '/plugins/advertwayCaptcha/refreshCaptcha.js', array('jquery', 'jquery-form'));
    else
        wp_enqueue_script('refreshCaptcha', WP_CONTENT_URL . '/mu-plugins/advertwayCaptcha/refreshCaptcha.js', array('jquery', 'jquery-form'));
  }


  /**
   * Check the solvemedia on WordPress MU
   * 
   * @param mixed $result
   * @return mixed  
   */
  function checkCaptchaWPMU($result)
  {
    // don't validate in the admin interface
    if(!is_admin())
    {
      //blogname in 2.6, blog_id prior to that
      if(isset($_POST['blog_id']) || isset($_POST['blogname']))
        return $result;

      //empty field
      if(empty($_POST['captcha_solution']) || $_POST['captcha_solution'] == '') 
      {
        $result['errors']->add('blank_captcha', $this->awcaptcha_opt['error_blank']);
        return $result;
      }

      //incorrect CAPTCHA
      if(!$this->awcaptcha->validate())
      {
        $result['errors']->add('captcha_wrong', $this->awcaptcha_opt['error_incorrect']);
        echo "<div class=\"error\">". $this->awcaptcha_opt['error_incorrect'] . "</div>";
      }
    }

    return $result;
  }
  
  /**
   *  Embeds the AdvertWayCaptcha widget into the comment form.
   *
   */
  function displayCaptchaInCommentForm()
  {
    global $user_ID;

    // set the minimum capability needed to skip the captcha if there is one
    if($this->awcaptcha_opt['bypass'] && $this->awcaptcha_opt['bypasslevel'])
      $needed_capability = $this->awcaptcha_opt['bypasslevel'];

    // skip the solvemedia display if the minimum capability is met
    if(($needed_capability && current_user_can($needed_capability)) || !$this->awcaptcha_opt['comments'])
      return;
    else
    {
      // Did the user fail to match the CAPTCHA? If so, let them know
      if(isset($_GET['rerror']))
        echo "<p class=\"awcaptcha-error\">" . $this->awcaptcha_opt['error_incorrect'] . "</p>";

      //modify the comment form for the solvemedia widget
      $comment_string = '<div id="awcaptcha-submit-btn-area"><br /></div>
        <script type="text/javascript">
            var submitBtn = document.getElementById("submit");
            if(submitBtn.parentNode != "undefined" && submitBtn.parentNode.tagName.toLowerCase() == "p")
              submitBtn = submitBtn.parentNode;
            submitBtn.parentNode.removeChild(submitBtn);
            document.getElementById("awcaptcha-submit-btn-area").appendChild(submitBtn);
            document.getElementById("submit").tabIndex = 6;
            if(typeof _savedcomment != "undefined"){
              document.getElementById("comment").value = _savedcomment;
            }
        </script>' .
        '<noscript>
          <style type="text/css">#submit {display:none;}</style>
          <input name="submit" type="submit" id="submit-alt" tabindex="6" value="Submit Comment"/>
        </noscript>';

      $this->awcaptcha->showCaptcha();  
      echo $comment_string;
    }
  }
  
  /**
   * Checks if the captcha solution was correct and sets an error session variable if not
   * @param mixed $comment_data
   * @return mixed $comment_data
   */
  function checkCommentCaptcha($comment_data)
  {
    global $user_ID;

    //set the minimum capability needed to skip the captcha if there is one
    if($this->awcaptcha_opt['bypass'] && $this->awcaptcha_opt['bypasslevel'])
      $needed_capability = $this->awcaptcha_opt['bypasslevel'];

    //skip the filtering if the minimum capability is met
    if(($needed_capability && current_user_can($needed_capability)) || !$this->awcaptcha_opt['comments'])
      return $comment_data;

    //don't check trackbacks/pingbacks
    if($comment_data['comment_type'] == '' )
    {
      if($this->awcaptcha->validate())
          return $comment_data;
      else
      {
        $this->savedError = $this->awcaptcha->getErrorMsg();
        add_filter('pre_comment_approved', create_function('$a', 'return \'spam\';'));
        return $comment_data;
      }
    }
    
    return $comment_data;
  }
  
  /*
   * If the awcaptcha guess was incorrect from checkCommentCaptcha, then redirect back to the comment form
   * @param string $location
   * @param object $comment
   * @return string $location
   */
  function relativeRedirect($location, $comment)
  {
    if($this->savedError != '')
    {
      //replace the '#comment-' chars on the end of $location with '#commentform'.
      $location = substr($location, 0,strrpos($location, '#')) .
                      ((strrpos($location, "?") === false) ? "?" : "&") .
                      'rcommentid=' . $comment->comment_ID .
                      '&rerror=1' .
                      '&rchash=' . $this->hashComment($comment->comment_ID) .
                      '#commentform';
    }
    return $location;
  }
  
  /*
   * If the captcha solution was incorrect from checkCommentCaptcha, then insert
   * saved comment text back in the comment form.
   * @return boolean
   */
  function savedComment()
  {
    if(!is_single() && !is_page())
      return;

    if($_GET['rcommentid'] && $_GET['rchash'] == $this->hashComment($_GET['rcommentid']))
    {
      $comment = get_comment($_GET['rcommentid']);

      $com = preg_replace('/([\\/\(\)\+\;\'\"])/e','\'%\'.dechex(ord(\'$1\'))', $comment->comment_content);
      $com = preg_replace('/\\r\\n/m', '\\\n', $com);

      echo '<script type="text/javascript">
              var _savedcomment =  "' . $com . '";
              _savedcomment = unescape(_savedcomment);
            </script>';

     wp_delete_comment($comment->comment_ID);
    }
  }
  
  function hashComment($id)
  {
    if(function_exists('wp_hash'))
      return wp_hash(AWCAPTCHA_WP_HASH_SALT . $id);
    else
      return md5(AWCAPTCHA_WP_HASH_SALT . $this->awcaptcha_opt['private_key'] . $id);
  }

  function addOptionsToAdminMenu()
  {
    if ($this->wpmu == 1 && is_site_admin()) 
    {
      add_submenu_page('wpmu-admin.php', 'AdvertWayCAPTCHA', 'AdvertWayCAPTCHA', 'manage_options', __FILE__, array($this, 'adminOptionsSubpanel'));
      add_options_page('AdvertWayCAPTCHA', 'AdvertWayCAPTCHA', 'manage_options', __FILE__, array($this,'adminSubpanel'));
    }
    else
    {
      add_options_page('AdvertWayCAPTCHA', 'AdvertWayCAPTCHA', 'manage_options', __FILE__, array($this,'adminSubpanel'));
    }
  }
  
  /*
   * Display an HTML <select> listing the capability options for disabling security
   * for registered users.
   * @param string $select_name slug to use in <select> id and name
   * @param string $checked_value selected value for dropdown, slug form.
   */

  function dropdownCapabilities($selectName, $checkedValue = "") 
  {
    $capabilityChoices = array(
      'Wszyscy zarejestrowani użytkownicy' => 'read',
      'edytować posty' => 'edit_posts',
      'publikować posty' => 'publish_posts',
      'moderować komentarze' => 'moderate_comments',
      'administrować stroną' => 'level_10'
    );
    
    echo '<select name="' . $selectName . '" id="' . $selectName . '">' . "\n";
    foreach ($capabilityChoices as $text => $capability)
    {
      $checked = $capability == $checkedValue ? ' selected="selected" ' : '';
      echo '\t <option value="' . $capability . '"' . $checked . ">$text</option> \n";
    }
    echo "</select> \n";
  }

  function adminSubpanel()
  {
    //Default values for the options array
    $optionarray_def = array(
      'public_key'          => '',
      'private_key'         => '',
      'bypasslevel'         => '3',
      'tabindex'            => '5',
      'comments'            => '1',
      'registration'        => '1',
      'contact_form_7'      => '1',
      'error_blank'         => '<strong>BŁĄD:</strong> Proszę uzupełnić AdvertWayCAPTCHA',
      'error_incorrect'     => '<strong>BŁĄD:</strong> Nieprawidłowe rozwiązanie AdvertWayCAPTCHA',
      'error_blank_cf7'     => "BŁĄD: Proszę uzupełnić AdvertWayCAPTCHA",
      'error_incorrect_cf7' => 'BŁĄD: Nieprawidłowe rozwiązanie',
    );

    if($this->wpmu != 1)
      add_option('awcaptcha', $optionarray_def, 'awcaptcha Options');

    //Check form submission and update options if no error occurred
    if(isset($_POST['submit']))
    {
      $pattern = '/<.*?>/';
      $substitue = '';
      $optionarray_update = array (
        'public_key'          => trim($_POST['awcaptcha_opt_public_key']),
        'private_key'         => trim($_POST['awcaptcha_opt_private_key']),
        'bypass'              => $_POST['bypass'],
        'bypasslevel'         => $_POST['bypasslevel'],
        'tabindex'            => $_POST['tabindex'],
        'comments'            => $_POST['comments'],
        'registration'        => $_POST['registration'],
        'contact_form_7'      => $_POST['contact_form_7'],
        'error_blank'         => $_POST['error_blank'],
        'error_incorrect'     => $_POST['error_incorrect'],
        'error_blank_cf7'     => preg_replace($pattern, $substitute, $_POST['error_blank']),
        'error_incorrect_cf7' => preg_replace($pattern, $substitute, $_POST['error_incorrect']),
      );
      
      //save updated options
      if($this->wpmu == 1)
        update_site_option('awcaptcha', $optionarray_update);
      else
        update_option('awcaptcha', $optionarray_update);
    }

    /* Get options */
    if($this->wpmu == 1)
      $optionarray_def = get_site_option('awcaptcha');
    else
      $optionarray_def = get_option('awcaptcha');
    ?>
  <div class="wrap">
  <h2>Opcje AdvertWayCAPTCHA</h2>
  <p>AdvertWayCaptcha oferuje usługę blokowania spamu na Twoim blogu, który dodatkowo może przynieść Ci zyski.</p>

  <p>Po więcej informacji zajrzyj na stronę <a href="http://www.advertway.pl">AdvertWay.pl</a>.</p>
  <p><strong>Uwaga</strong>: Jeżeli używasz jakiegoś pluginu cache'ującego, prawdopodobnie potrzebne będzie wyczyszczenie cache'u, aby zmiany zostały zastosowane.</p>

  <form name="form1" method="post" action="<?php echo $_SERVER['REDIRECT_SCRIPT_URI'] . '?page=' . plugin_basename(__FILE__); ?>&updated=true">
    <div class="submit">
      <input type="submit" name="submit" value="Zapisz zmiany &raquo;" />
    </div>

    <table class="form-table">
      <tr valign="top">
      <th scope="row">Klucze witryny</th>
        <td>
          AdvertWayCAPTCHA wymaga do działania dwóch kluczy: "publicznego" i "prywatnego".
          <br />
          Możesz je zdobyć w panelu wydawcy pod adresem <a href="http://publisher.advertway.pl">http://publisher.advertway.pl</a>.
          <br />
          <p class="re-keys">
            <label class="which-key" for="awcaptcha_opt_public_key">Klucz publiczny:</label>
            <input name="awcaptcha_opt_public_key" id="awcaptcha_opt_public_key" size="40" value="<?php  echo $optionarray_def['public_key']; ?>" />
            <br />
            <label class="which-key" for="awcaptcha_opt_private_key">Klucz prywatny:</label>
            <input name="awcaptcha_opt_private_key" id="awcaptcha_opt_private_key" size="40" value="<?php  echo $optionarray_def['private_key']; ?>" />
            <br />
          </p>
        </td>
      </tr>
      <tr valign="top">
      <th scope="row">Opcje komentarzy</th>
        <td>
          <input type="checkbox" name="comments" id="comments" value="1" <?php if($optionarray_def['comments'] == true){echo 'checked="checked"';} ?> /> <label for="comments">Włącz AdvertWayCAPTCHA w komentarzach</label>
          <br />
          <div class="theme-select">
            <input type="checkbox" id="bypass" name="bypass" <?php if($optionarray_def['bypass'] == true){echo 'checked="checked"';} ?>/>
            <label name="bypass" for="bypass">Ukryj AdvertWayCAPTCHA dla <strong>zarejestrowanych</strong> użytkowników, którzy mogą:</label>
            <?php $this->dropdownCapabilities('bypasslevel', $optionarray_def['bypasslevel']); // <select> of capabilities ?>
          </div>
          <br />
        </td>
      </tr>
      <?php if($this->wpmu == 1 || $this->wpmu == 0): ?>
        <tr valign="top">
        <th scope="row">Opcje rejestracji</th>
          <td>
            <input type="checkbox" name="registration" id="registration" value="1" <?php if($optionarray_def['registration'] == true){echo 'checked="checked"';} ?> /> <label for="registration">Włącz AdvertWayCAPTCHA w formularzu rejestracyjnym</label>
          </td>
        </tr>
        <tr valign="top">
        <th scope="row">Opcje dla "Contact Form 7"</th>
          <td>
            <input type="checkbox" name="contact_form_7" id="contact_form_7" value="1" <?php if($optionarray_def['contact_form_7'] == true){echo 'checked="checked"';} ?> /> <label for="contact_form_7">Włącz AdvertWayCAPTCHA dla wtyczki "Contact Form 7".</label>
          </td>
        </tr>
      <?php endif; ?>
      <tr valign="top">
      <th scope="row">Komunikaty błędów</th>
        <td>
          <p>Poniższe komunikaty będą wyświetlane, gdy użytkownik nie wpisze lub wprowadzi błędne rozwiązanie AdvertWayCAPTCHA</p>
          <p class="re-keys">
            <label class="which-key" for="error_blank">Rozwiąznie nie wpisane przez użytkownika:</label>
            <input name="error_blank" id="error_blank" size="80" value="<?php echo $optionarray_def['error_blank']; ?>" />
            <br />
            <label class="which-key" for="error_incorrect">Niepoprawne rozwiązanie:</label>
            <input name="error_incorrect" id="error_incorrect" size="80" value="<?php echo $optionarray_def['error_incorrect']; ?>" />
          </p>
        </td>
      </tr>
    </table>
    <div class="submit">
      <input type="submit" name="submit" value="Zapisz zmiany &raquo;" />
    </div>

  </form>
    <p class="copyright">&copy; Copyright 2011&nbsp;&nbsp;<a href="http://www.advertway.pl">AdvertWay.pl</a></p>
</div>
<?php
  }  
  
  function pluginWarning() 
  {
    $path = plugin_basename(__FILE__);
    echo '<div id="awcaptcha-warning" class="updated fade-ff0000"><p><strong>Plugin AdvertWayCAPTCHA jest nieaktywny.</strong> Musisz <a href="options-general.php?page=' . $path . '">wprowadzić swój klucz prywatny i publiczny.</a></p></div>
          <style type="text/css">#adminmenu {margin-bottom: 5em;}</style>';
  }
}

$awcaptchaPlugin = new AdvertWayCaptchaWPPlugin();

if(!defined('WP_CONTENT_URL'))
  define('WP_CONTENT_URL', get_option('siteurl') . '/wp-content');

//Loading advertway lib
if($awcaptchaPlugin->getWPMU() == 1)
 require_once(dirname(__FILE__) . '/advertway/advertway_captcha_lib.php');
else
 require_once(dirname(__FILE__) . '/advertway_captcha_lib.php');

if(file_exists(WP_PLUGIN_DIR . '/contact-form-7/includes/shortcodes.php'))
  require_once(WP_PLUGIN_DIR . '/contact-form-7/includes/shortcodes.php');
if(file_exists(WP_PLUGIN_DIR . '/contact-form-7/includes/taggenerator.php'))
  require_once(WP_PLUGIN_DIR . '/contact-form-7/includes/taggenerator.php');

//awcaptchaApi initialize
$awcaptchaPlugin->awCaptchaInit();

register_deactivation_hook(__FILE__, array($awcaptchaPlugin, 'deletePreferences'));
add_action('init', array($awcaptchaPlugin, 'enqueueJS'));

// include the stylesheet in typical pages to style hidden emails
add_action('wp_head', array($awcaptchaPlugin, 'includeStylesheet')); 
// include stylesheet to style options page
add_action('admin_head', array($awcaptchaPlugin, 'includeStylesheet'));
// include the login div styling
add_action('login_head', array($awcaptchaPlugin, 'includeRegistrationStylesheet'));

//Display awcaptcha on registration page
if($awcaptchaPlugin->getWPMU() != 1)
  add_action('register_form', array($awcaptchaPlugin, 'displayCaptcha'));
else
  add_action('signup_extra_fields', array($awcaptchaPlugin, 'displayCaptcha'));


add_action('plugins_loaded', 'awcaptcha_appendToCF7');
function awcaptcha_appendToCF7()
{
  global $awcaptchaPlugin;
  //Display awcaptcha in Contact Form 7 form
  if(function_exists('wpcf7_add_shortcode') && $awcaptchaPlugin->awcaptcha_opt['contact_form_7'])
    wpcf7_add_shortcode('awcaptcha', array($awcaptchaPlugin, 'displayCaptcha'), true);
  //check captcha in Contact Form 7 form
  add_filter('wpcf7_validate_awcaptcha', array($awcaptchaPlugin, 'checkCaptcha'), 10, 2);
}

//Check awcaptcha in registration form
if($awcaptchaPlugin->awcaptcha_opt['registration'])
{
  if($awcaptchaPlugin->getWPMU() == 1)
    add_filter('wpmu_validate_user_signup', array($awcaptchaPlugin, 'checkCaptchaWPMU'));
  elseif($awcaptchaPlugin->getWPMU() == 0)
    add_filter('registration_errors', array($awcaptchaPlugin, 'checkCaptcha'));
}

//display awcaptcha in comment form
add_action('comment_form', array($awcaptchaPlugin, 'displayCaptchaInCommentForm'));
//check awcaptcha in comment form
add_filter('preprocess_comment', array($awcaptchaPlugin, 'checkCommentCaptcha'),0);
add_filter('wp_head', array($awcaptchaPlugin, 'savedComment'), 0);
add_filter('comment_post_redirect', array($awcaptchaPlugin, 'relativeRedirect'),0,2);

//adds AdvertWayCAPTCHA to admin menu
add_action('admin_menu', array($awcaptchaPlugin, 'addOptionsToAdminMenu'));

if(!($awcaptchaPlugin->awcaptcha_opt['public_key'] && $awcaptchaPlugin->awcaptcha_opt['private_key']) && !isset($_POST['submit']) )
{
  if(($awcaptchaPlugin->getWPMU() == 1 && is_site_admin()) || $awcaptchaPlugin->getWPMU() != 1)
    add_action('admin_footer', array($awcaptchaPlugin, 'pluginWarning'));
}